<?php
namespace OrganizationsManagement\Controller;

use Exception\CustomException;
use Zeedhi\Framework\Controller\Simple;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Util\Messages;
use Util\InstanceProvider;
use OrganizationsManagement\Util\Utilities;
use OrganizationsManagement\Service;

class ScoreReport
{

    /** @var Service\ScoreReport $serviceScoreReport */
    protected $serviceScoreReport;

    public function __construct(Service\ScoreReport $serviceScoreReport){
        $this->serviceScoreReport = $serviceScoreReport;
    }

    public function findCalls(Request $request, Response $response)
    {
        ini_set('max_execution_time', '900');
        set_time_limit(900);
        try{
            /** @var $request Request\Filter*/
            $conditions     = $request->getFilterCriteria()->getConditions();
            $page           = $request->getFilterCriteria()->getPage();
            $pageSize       = $request->getFilterCriteria()->getPageSize();
            $orderBy        = $request->getFilterCriteria()->getOrderBy();
            $orgId          = Utilities::getValueInFilterDataSourceByName($conditions,'ORG_ID', true);
            $groupByLabel   = Utilities::getValueInFilterDataSourceByName($conditions,'GROUP_BY_LABEL', true);
            $mainType       = Utilities::getValueInFilterDataSourceByName($conditions,'MAIN_TYPE_CALL', true);
            foreach($conditions as $i => $condition){
                if($condition['columnName'] == "REFERENCE"){
                    $conditions[$i]["value"] = str_replace("'","",$condition["value"]);
                }
            }
            $calls          = $this->serviceScoreReport->findCalls($mainType, $conditions, $orderBy, $page, $pageSize,
                $groupByLabel, $orgId);
            $response->addDataSet(new DataSet('findCalls', $calls));
        } catch(\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }

    public function findAllOperadorRelationshipCliente(Request\Filter $request, Response $response){
        try{
            $request->getFilterCriteria()->addCondition("IDATIVO", DataSource\FilterCriteria::EQ, "S");
            $condition = $request->getFilterCriteria()->getConditions();
            $dataSet = $this->serviceScoreReport->findAllOperadorRelationshipCliente($condition);
            $response->addDataSet(new DataSet('findAllOperadorRelationshipCliente', $dataSet));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }

    public function findAllOperadorResponsible(Request\Filter $request, Response $response){
        try{
            $page      = $request->getFilterCriteria()->getPage();
            $pageSize  = $request->getFilterCriteria()->getPageSize();
            $condition = $request->getFilterCriteria()->getConditions();
            $dataSet = $this->serviceScoreReport->findAllOperadorResponsible($condition, $page, $pageSize);
            $response->addDataSet(new DataSet('findAllOperadorResponsible', $dataSet));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }

    public function faturamento(Request\Filter $request, Response $response)
    {
        try {
            $conditions = $request->getFilterCriteria()->getConditions();
            $orderBy = $request->getFilterCriteria()->getOrderBy();
            $page = $request->getFilterCriteria()->getPage();
            $pageSize = $request->getFilterCriteria()->getPageSize();
            $nrparcnegocio   = Utilities::getValueInFilterDataSourceByName($conditions,'NRPARCNEGOCIO', true);
            $dtinicompet   = Utilities::getValueInFilterDataSourceByName($conditions,'DTINICOMPET', false);
            $dtfincompet   = Utilities::getValueInFilterDataSourceByName($conditions,'DTFINCOMPET', false);
            $resultSet  = $this->serviceScoreReport->faturamento($conditions, $nrparcnegocio, $dtinicompet, $dtfincompet, $page, $pageSize, $orderBy);
            $response->addDataSet(new DataSet('faturamento', $resultSet));
        } catch (CustomException $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }
    public function getContratosForScore(Request\Filter $request, Response $response)
    {
        try {
            $conditions = $request->getFilterCriteria()->getConditions();
            $nrparcnegocio   = Utilities::getValueInFilterDataSourceByName($conditions,'NRPARCNEGOCIO', true);
            $resultSet  = $this->serviceScoreReport->getContratosForScore($nrparcnegocio);
            $response->addDataSet(new DataSet('contratosNumber', $resultSet));
        } catch (CustomException $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }

    public function getItensContrato(Request\Filter $request, Response $response)
    {
        try {
            $conditions = $request->getFilterCriteria()->getConditions();
            $nrfaturacont   = Utilities::getValueInFilterDataSourceByName($conditions,'NRFATURACONT', true);
            $nrorg   = Utilities::getValueInFilterDataSourceByName($conditions,'NRORG', true);
            $nrparcnegocio   = Utilities::getValueInFilterDataSourceByName($conditions,'NRPARCNEGOCIO', true);
            $resultSet  = $this->serviceScoreReport->getItensContrato($nrfaturacont, $nrorg, $nrparcnegocio,$conditions);
            $response->addDataSet(new DataSet('itensContrato', $resultSet));
        } catch (CustomException $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }

    public function getContratos(Request\Filter $request, Response $response)
    {
        try {
            $orderBy = $request->getFilterCriteria()->getOrderBy();
            $page = $request->getFilterCriteria()->getPage();
            $pageSize = $request->getFilterCriteria()->getPageSize();
            $conditions = $request->getFilterCriteria()->getConditions();
            $nrparcnegocio   = Utilities::getValueInFilterDataSourceByName($conditions,'NRPARCNEGOCIO', true);
            $isFilter   = Utilities::getValueInFilterDataSourceByName($conditions,'IS_FILTER', true);
            $resultSet  = $this->serviceScoreReport->getContratos($conditions, $nrparcnegocio, $isFilter, $page, $pageSize, $orderBy);
            $response->addDataSet(new DataSet('contratos', $resultSet));
        } catch (CustomException $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }

    public function getItemsOrcamento(Request\Filter $request, Response $response)
    {
        try {
            $conditions = $request->getFilterCriteria()->getConditions();
            $nrorg   = Utilities::getValueInFilterDataSourceByName($conditions,'NRORG', true);
            $idadicitemorc   = Utilities::getValueInFilterDataSourceByName($conditions,'IDADICITEMORC', true);
            $nrseqcontrato   = Utilities::getValueInFilterDataSourceByName($conditions,'NRSEQCONTRATO', true);
            $nrseqorcamento   = Utilities::getValueInFilterDataSourceByName($conditions,'NRSEQORCAMENTO', true);
            $resultSet  = $this->serviceScoreReport->getItemsOrcamento($nrorg, $idadicitemorc, $nrseqcontrato, $nrseqorcamento);
            $response->addDataSet(new DataSet('getItemsOrcamento', $resultSet));
        } catch (CustomException $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }

    public function getNegociacaoContratoCompleto(Request\Filter $request, Response $response)
    {
        try {
            $filters = $request->getFilterCriteria()->getConditions();
            $nrseqcontrato   = Utilities::getValueInFilterDataSourceByName($filters,'NRSEQCONTRATO', true);
            $resultSet  = $this->serviceScoreReport->getNegociacaoContratoCompleto($nrseqcontrato, $filters);
            $response->addDataSet(new DataSet('getItemsOrcamento', $resultSet));
        } catch (CustomException $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }

    public function getCompetenciaNegoCont(Request\Filter $request, Response $response)
    {
        try {
            $conditions = $request->getFilterCriteria()->getConditions();
            $resultSet  = $this->serviceScoreReport->getCompetenciaNegoCont($conditions);
            $response->addDataSet(new DataSet('getItemsOrcamento', $resultSet));
        } catch (CustomException $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }

    public function getParcelanegocont(Request\Filter $request, Response $response)
    {
        try {
            $conditions = $request->getFilterCriteria()->getConditions();
            $resultSet  = $this->serviceScoreReport->getParcelanegocont($conditions);
            $response->addDataSet(new DataSet('getItemsOrcamento', $resultSet));
        } catch (CustomException $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }

    public function getDescperiodonegocont_negociacaocontrato(Request\Filter $request, Response $response)
    {
        try {
            $conditions = $request->getFilterCriteria()->getConditions();
            $resultSet  = $this->serviceScoreReport->getDescperiodonegocont_negociacaocontrato($conditions);
            $response->addDataSet(new DataSet('getItemsOrcamento', $resultSet));
        } catch (CustomException $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }

    public function getElements(Request\Filter $request, Response $response)
    {
        try {
            $conditions = $request->getFilterCriteria()->getConditions();
            $nrorg   = Utilities::getValueInFilterDataSourceByName($conditions,'NRORG', false);
            $idescolhido   = Utilities::getValueInFilterDataSourceByName($conditions,'IDESCOLHIDO', false);
            $nrseqorcamento   = Utilities::getValueInFilterDataSourceByName($conditions,'NRSEQORCAMENTO', false);
            $resultSet  = $this->serviceScoreReport->getElements($nrorg, $idescolhido, $nrseqorcamento, $conditions);
            $response->addDataSet(new DataSet('getElements', $resultSet));
        } catch (CustomException $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }

    public function findProdutosContratados(DTO\Request\Filter $request, DTO\Response $response) {
        try {
            $conditions  = $request->getFilterCriteria()->getConditions();
            $nrparcnegocio  = Utilities::getValueInFilterDataSourceByName($conditions, 'NRPARCNEGOCIO', true);
            $data = $this->serviceScoreReport->findProdutosContratados($conditions, $nrparcnegocio);
            $response->addDataSet(new DataSource\DataSet('produtosContratados', $data));
        } catch(\Exception $ex){
            $response->addMessage(new DTO\Response\Message($ex->getMessage()));
            $response->setError(new DTO\Response\Error($ex->getMessage(), $ex->getCode()));
        }
    }

    public function getOrganizationScore(DTO\Request\Filter $request, DTO\Response $response) {
        try {
            $filters = $request->getFilterCriteria()->getConditions();
            $page = $request->getFilterCriteria()->getPage();
            $pageSize = $request->getFilterCriteria()->getPageSize();
            $orderBy = $request->getFilterCriteria()->getOrderBy();
            $isPrimary =  Utilities::getValueInFilterDataSourceByName($filters, 'IS_PRIMARY', true);
            $data = $this->serviceScoreReport->getOrganizationScore($filters, $page, $pageSize, $orderBy, null, $isPrimary);
            $response->addDataSet(new DataSource\DataSet('getOrganizationScore', $data));
        } catch(\Exception $ex){
            $response->addMessage(new DTO\Response\Message($ex->getMessage()));
            $response->setError(new DTO\Response\Error($ex->getMessage(), $ex->getCode()));
        }
    }

    public function autoUpdateNrscores(DTO\Request $request, DTO\Response $response) {
        try {
            ini_set("memory_limit", "2048M");
            ini_set('max_execution_time', '3600');
            set_time_limit(3600);
            $this->serviceScoreReport->autoUpdateNrscores();
        } catch(\Exception $ex){
            $response->addMessage(new DTO\Response\Message($ex->getMessage()));
            $response->setError(new DTO\Response\Error($ex->getMessage(), $ex->getCode()));
        }
    }

    public function saveScoreReport(Request\Dataset $request, Response $response){
        try{
            $data = $request->getDataSet()->getRows()[0];
            $score          = empty($data['SCORE']) ? null : $data['SCORE'];
            $alert          = empty($data['ALERT']) ? null : $data['ALERT'];
            $director       = empty($data['DIRECTOR']) ? null : $data['DIRECTOR'];
            $responsible    = empty($data['IT_RESPONSIBLE']) ? null : $data['IT_RESPONSIBLE'];
            $godfather      = empty($data['GODFATHER']) ? null : $data['GODFATHER'];
            $manager        = empty($data['MANAGER']) ? null : $data['MANAGER'];
            $inProject      = empty($data['IN_PROJECT']) ? null : $data['IN_PROJECT'];
            $nrorg          = empty($data['NRORG']) ? null : $data['NRORG'];
            $idpercpimp     = empty($data['IDPERCPIMP']) ? null : $data['IDPERCPIMP'];
            $saved = $this->serviceScoreReport->saveScoreReport($score, $alert, $director, $responsible, $godfather, $manager, $inProject, $nrorg,$idpercpimp);
            $response->addDataSet(new DataSet('saved', [$saved]));
        } catch(\Exception $ex){
            $response->addMessage(new DTO\Response\Message($ex->getMessage()));
            $response->setError(new DTO\Response\Error($ex->getMessage(), $ex->getCode()));
        }
    }

    public function findCallsByType(DTO\Request\Filter $request, DTO\Response $response) {
        try{
            $conditions  = $request->getFilterCriteria()->getConditions();
            $nrorg  = Utilities::getValueInFilterDataSourceByName($conditions, 'NRORG', true);
            $data = $this->serviceScoreReport->findCallsByType($nrorg);
            $response->addDataSet(new DataSource\DataSet('callsNumber', [$data]));
        } catch(\Exception $ex){
            $response->addMessage(new DTO\Response\Message($ex->getMessage()));
            $response->setError(new DTO\Response\Error($ex->getMessage(), $ex->getCode()));
        }
    }

    public function escopoOrcamento(DTO\Request\Filter $request, DTO\Response $response) {
        try{
            $conditions  = $request->getFilterCriteria()->getConditions();
            $orderBy = $request->getFilterCriteria()->getOrderBy();
            $page = $request->getFilterCriteria()->getPage();
            $pageSize = $request->getFilterCriteria()->getPageSize();
            $data = $this->serviceScoreReport->getEscopoTIpoOrcamento($conditions, $page, $pageSize, $orderBy);
            $response->addDataSet(new DataSource\DataSet('escopoOrcamento', $data));
        } catch(\Exception $ex){
            $response->addMessage(new DTO\Response\Message($ex->getMessage()));
            $response->setError(new DTO\Response\Error($ex->getMessage(), $ex->getCode()));
        }
    }
    public function findFinanceiro(Request\Filter $request, Response $response)
    {
        try {
            $conditions = $request->getFilterCriteria()->getConditions();
            $orderBy = $request->getFilterCriteria()->getOrderBy();
            $page = $request->getFilterCriteria()->getPage();
            $pageSize = $request->getFilterCriteria()->getPageSize();

            $nrparcnegocio   = Utilities::getValueInFilterDataSourceByName($conditions,'NRPARCNEGOCIO', true);

            $dtemissao   = Utilities::getValueInFilterDataSourceByName($conditions,'DTEMISSAO', true);
            $dtemissao = empty($dtemissao) ? null :$dtemissao;

            $dtvencimento   = Utilities::getValueInFilterDataSourceByName($conditions,'DTATUAVENREC', true);
            $dtvencimento = empty($dtvencimento) ? null :$dtvencimento;

            $dtpagamento   = Utilities::getValueInFilterDataSourceByName($conditions,'DTBAIXAREC', true);
            $dtpagamento = empty($dtpagamento) ? null :$dtpagamento;

            $vencboleto   = Utilities::getValueInFilterDataSourceByName($conditions,'VENCBOLETO', true);
            $resultSet  = $this->serviceScoreReport->financeiro($conditions, $orderBy, $page, $pageSize, $nrparcnegocio, $vencboleto, $dtemissao, $dtvencimento, $dtpagamento);
            $response->addDataSet(new DataSet('faturamento', $resultSet));
        } catch (CustomException $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }


}
