<?php
namespace OrganizationsManagement\Controller;

use OrganizationsManagement\Util\InstanceProvider;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource;
use OrganizationsManagement\Util\Utilities;
use Zeedhi\Framework\DTO\Response;

class StructureUpdate
{
    /**
     * @param DTO\Request\DataSet $request
     * @param DTO\Response $response
     */
    public function executeScript(DTO\Request\Dataset $request, DTO\Response $response){
        try {
            $row = $request->getDataSet()->getRows()[0];
            $organizationId = $row['ORGANIZATION_ID'];
            $script         = $row['SCRIPT'];
            $scriptResult    = InstanceProvider::getService()->getStructureUpdate()->executeScript($organizationId, $script);
            $response->addDataSet(new DataSource\DataSet('scriptResult', $scriptResult));
        } catch(\Exception $ex) {
            $response->setError(new DTO\Response\Error($ex->getMessage(), $ex->getCode()));
        }
    }

    /**
     * @param DTO\Request\DataSet $request
     * @param DTO\Response $response
     */
    public function saveStructureUpdateForOrgs(DTO\Request\Dataset $request, DTO\Response $response){
        try {
            $row = $request->getDataSet()->getRows()[0];
            $structureUpdateId = $row['STRUCTURE_UPDATE_ID'];
            $structureUpdateOrgId = $row['STRUCTURE_UPDATE_ORG_ID'];
            $organizationId = $row['ORGANIZATION_ID'];
            $log = $row['LOG'][0];
            $logDir = $log['path'];
            $idResultScript = $row['IDRESULT'];

            $savedRegisterId = InstanceProvider::getService()->getStructureUpdate()->saveStructureUpdateForOrgs($logDir, $organizationId, $idResultScript, $structureUpdateId, $structureUpdateOrgId);
            $response->addDataSet(new DataSource\DataSet('savedStructureUpdateOrgs', $savedRegisterId));
        } catch(\Exception $ex) {
            $response->setError(new DTO\Response\Error($ex->getMessage(), $ex->getCode()));
        }
    }

    public function findTicketMovDeskDeploy(Dto\Request\Filter $request, Response $response) {
        try {
            $validate  =InstanceProvider::getService()->getStructureUpdate()->findTicketMovDeskDeploy();
            $response->addDataSet(new DataSet('findTicketMovDeskDeploy', $validate));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage()));
            $response->setError(new Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }

    public function saveStructureUpdate(DTO\Request\Dataset $request, DTO\Response $response){
        try {
            $row = $request->getDataSet()->getRows()[0];
            $organizations   = $row['ORGANIZATION_IDS'];
            $script          = $row['SCRIPT'];
            $description     = isset($row['DSATUESTRUTURA']) ? $row['DSATUESTRUTURA'] : null;
            $scriptSuc       = $row['SCRIPT_SUC'];
            $scriptErr       = $row['SCRIPT_ERR'];
            $ticketId        = $row['TICKET_ID'];
            $idResultScript = $row['IDRESULT'];


            $savedRegisterId = InstanceProvider::getService()->getStructureUpdate()->saveStructureUpdate($script, $description, $ticketId, $idResultScript, $scriptSuc, $scriptErr, $organizations);
            $response->addDataSet(new DataSource\DataSet('savedStructureUpdate', $savedRegisterId));
        } catch(\Exception $ex) {
            $response->setError(new DTO\Response\Error($ex->getMessage(), $ex->getCode()));
        }
    }

    public function saveStructureUpdateResults(DTO\Request\Dataset $request, DTO\Response $response){
        try {
            $row = $request->getDataSet()->getRows()[0];
            $structureUpdateId = $row['STRUCTURE_UPDATE_ID'];
            $scriptSuc       = $row['SCRIPT_SUC'];
            $scriptErr       = $row['SCRIPT_ERR'];
            if(is_array($scriptSuc)){
                $allScriptSuc    = implode("\n/\n", $scriptSuc);
            } else{
                $allScriptSuc = $scriptSuc;
            }
            if(is_array($scriptErr)){
                $allScriptErr    = implode("\n/\n", $scriptErr);
            } else{
                $allScriptErr = $scriptErr;
            }
            $savedRegisterId = InstanceProvider::getService()->getStructureUpdate()->saveStructureUpdateResults($structureUpdateId, $allScriptSuc, $allScriptErr);
            $response->addDataSet(new DataSource\DataSet('savedStructureUpdate', $savedRegisterId));
        } catch(\Exception $ex) {
            $response->setError(new DTO\Response\Error($ex->getMessage(), $ex->getCode()));
        }
    }

    /**
     * @param DTO\Request\Filter $request
     * @param DTO\Response $response
     */
    public function findStructureUpdate(DTO\Request\Filter $request, DTO\Response $response){
        try{
            $filters   = $request->getFilterCriteria()->getConditions();
            $page      = $request->getFilterCriteria()->getPage();
            $pageSize  = $request->getFilterCriteria()->getPageSize();
            $orderBy   = $request->getFilterCriteria()->getOrderBy();
            $scriptId = Utilities::getValueInFilterDataSourceByName($filters, 'SCRIPT_ID', true);
            $registers = InstanceProvider::getService()->getStructureUpdate()->findStructureUpdate($filters, $page, $pageSize, $orderBy, $scriptId ? $scriptId : null);
            $response->addDataSet(new DataSource\DataSet('structureUpdate', $registers));
        } catch(\Exception $ex){
            $response->setError(new DTO\Response\Error($ex->getMessage(), 500));
            $response->addMessage(new DTO\Response\Message($ex->getMessage()));
        }
    }

    public function validateTicket(DTO\Request\Filter $request, DTO\Response $response){
        try{
            $filters = $request->getFilterCriteria()->getConditions();
            $ticket = Utilities::getValueInFilterDataSourceByName($filters, 'TICKET', true);
            $valid = InstanceProvider::getService()->getStructureUpdate()->validateTicket($ticket);
            $response->addDataSet(new DataSource\DataSet('data', [$valid]));
        } catch(\Exception $ex){
            $response->setError(new DTO\Response\Error($ex->getMessage(), 500));
            $response->addMessage(new DTO\Response\Message($ex->getMessage()));
        }
    }

    public function validateTicketIfScriptAlreadySucceeded(DTO\Request\Filter $request, DTO\Response $response){
        try{
            $filters = $request->getFilterCriteria()->getConditions();
            $ticket = Utilities::getValueInFilterDataSourceByName($filters, 'TICKET', true);

            $result = InstanceProvider::getService()->getStructureUpdate()->validateTicketIfScriptAlreadySucceeded($ticket);

            $response->addDataSet(new DataSource\DataSet('data', [$result]));
        } catch(\Exception $ex){
            $response->setError(new DTO\Response\Error($ex->getMessage(), $ex->getCode()));
        }
    }

    public function validateSucceededScripts(DTO\Request\Filter $request, DTO\Response $response){
        try{
            $filters = $request->getFilterCriteria()->getConditions();
            $ticket = Utilities::getValueInFilterDataSourceByName($filters, 'TICKET', true);

            $result = InstanceProvider::getService()->getStructureUpdate()->validateSucceededScripts($ticket);
            $response->addDataSet(new DataSource\DataSet('data', [$result]));
        } catch(\Exception $ex){
            $response->setError(new DTO\Response\Error($ex->getMessage(), $ex->getCode()));
        }
    }

    public function findErrorOrPartialOrganizations(DTO\Request\Filter $request, DTO\Response $response){
        try{
            $filters = $request->getFilterCriteria()->getConditions();
            $id = Utilities::getValueInFilterDataSourceByName($filters, 'ID', true);
            $result = Utilities::getValueInFilterDataSourceByName($filters, 'RESULT', true);

            $data = InstanceProvider::getService()->getStructureUpdate()->findErrorOrPartialOrganizations($id, $result);
            $response->addDataSet(new DataSource\DataSet('data', $data));
        } catch(\Exception $ex){
            $response->setError(new DTO\Response\Error($ex->getMessage(), $ex->getCode()));
        }
    }


    /**
     * @param DTO\Request\DataSet $request
     * @param DTO\Response $response
     */
    public function downloadStructureUpdateLog(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            $row = $request->getDataSet()->getRows()[0];
            $fileDir  = $row['FILE_DIR'];
            $fileName = $row['FILE_NAME'];
            InstanceProvider::getService()->getStructureUpdate()->downloadStructureUpdateLog($fileDir, $fileName);
        } catch(\Exception $ex){
            $response->addMessage(new DTO\Response\Message($ex->getMessage()));
            $response->setError(new DTO\Response\Error($ex->getMessage(), $ex->getCode()));
        }
    }

    public function downloadStructureUpdateScripts(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            $row = $request->getDataSet()->getRows()[0];
            $fileName = $row['FILE_NAME'];
            $ticket = $row['TICKET_ID'];
            $fileDir = Utilities::getTempLogPathParameter();

            InstanceProvider::getService()->getStructureUpdate()->downloadStructureUpdateScripts($fileName, $fileDir, $ticket);
        } catch(\Exception $ex){
            $response->setError(new DTO\Response\Error($ex->getMessage(), $ex->getCode()));
        }
    }
}
