<?php
namespace OrganizationsManagement\Service;
use Doctrine\ORM\EntityManager;
use OrganizationsManagement\Util\Utilities;
use OrganizationsManagement\Util\Messages as Messages;
use OrganizationsManagement\Factory;
use OrganizationsManagement\Repository;
use Teknisa\Model\Teknisa\Repositories;
use Teknisa\Model\Integracao;
use Teknisa\Libs\Util\Environment as TeknisaEnvironment;

class LayoutExportation
{
    /** @var Integracao\Repositories\LayoutExp **/
    protected $repositoryLayoutexp;
    
    /** @var Integracao\Repositories\Tablayoutexp **/
    protected $repositoryTablayoutexp;

    /** @var Utilities **/
    protected $utilities;

    /** @var EntityManager **/
    protected $entityManager;

    /** @var TeknisaEnvironment **/
    protected $environment;

    /** @var Factory\LayoutExportation **/
    protected $factoryLayoutExportation;
    
    /** @var Factory\TableLayoutExportation **/
    protected $factoryTableLayoutExportation;

    /** @var Repository\LayoutExportation */
    private $repositoryLayoutExportation;

    public function __construct(EntityManager $entityManager, Utilities $utilities, TeknisaEnvironment $environment,
                                Factory\LayoutExportation $factoryLayoutExportation, Factory\TableLayoutExportation $factoryTableLayoutExportation,
                                Repository\LayoutExportation $repositoryLayoutExportation){
        $this->entityManager = $entityManager;
        $this->utilities = $utilities;
        $this->environment = $environment;
        $this->factoryLayoutExportation = $factoryLayoutExportation;
        $this->factoryTableLayoutExportation = $factoryTableLayoutExportation;
        $this->repositoryLayoutexp = $this->entityManager->getRepository(Utilities::INTEGRACAO_ENTITY_DIR . 'Layoutexp');
        $this->repositoryTablayoutexp = $this->entityManager->getRepository(Utilities::INTEGRACAO_ENTITY_DIR . 'Tablayoutexp');
        $this->repositoryLayoutExportation = $repositoryLayoutExportation;
    }
    
    public function saveLayout($row){
        $layoutName = $row['NAME'];
        $layoutActive = $row['ACTIVE'];
        $layoutId = isset($row['ID']) ? $row['ID'] : null;
        $dateNow = new \DateTime();
        $orgId = $this->environment->getCurrentOrganizationId();
        $userId = $this->environment->getCurrentUserId();
        $connection = $this->entityManager->getConnection();
        $layoutId ? $layout = $this->repositoryLayoutexp->findOneBy(array('cdlayoutexp' => $layoutId)) : $layout=null;
        try {
            $connection->beginTransaction();
            $savedLayout = $this->factoryLayoutExportation->save($layout, $userId, $dateNow, $layoutName, $orgId, $layoutActive);
            $this->entityManager->persist($savedLayout);
            $this->entityManager->flush();
            $connection->commit();
            return array("ID"=>$savedLayout->getCdlayoutexp());
        } catch (\Exception $ex){
            $connection->rollBack();
            throw $ex;
        }
    }
    
    public function removeLayout($layoutId){
        $connection = $this->entityManager->getConnection();
        $layout = $this->repositoryLayoutexp->findOneBy(array('cdlayoutexp' => $layoutId));
        try {
            $connection->beginTransaction();
            if ($layout) {
                $tablesOfLayout = $this->repositoryTablayoutexp->findBy(array('cdlayoutexp' => $layoutId));
                if ($tablesOfLayout){
                    foreach($tablesOfLayout as $table){
                        $this->entityManager->remove($table);
                    }
                    $this->entityManager->flush();
                }
                $this->entityManager->remove($layout);
                $this->entityManager->flush();
                $connection->commit();
            } else {
                throw new Exception(Messages::ERR_NOT_REMOVED_EXPLANATION .'o layout não existe!');
            }
        } catch (\Exception $ex){
            $connection->rollBack();
            throw $ex;
        }
    }
    
    public function removeLayoutTables($row){
        $layoutId = $row['LAYOUT_ID'];
        $tableName = $row['NAME'];
        $connection = $this->entityManager->getConnection();
        $layoutTables = $this->repositoryTablayoutexp->findOneBy(array('cdlayoutexp' => $layoutId, 'nmtabelabd'=> $tableName));
        try {
            $connection->beginTransaction();
            if ($layoutTables) {
                $this->entityManager->remove($layoutTables);
                $this->entityManager->flush();
                $connection->commit();
            } else {
                throw new \Exception(Messages::ERR_NOT_REMOVED_EXPLANATION .'a tabela não está associada ao layout!');
            }
        } catch (\Exception $ex){
            $connection->rollBack();
            throw $ex;
        }
    }
    
    public function getLayoutExportation($filters){
        return $this->repositoryLayoutexp->getLayoutExportation(Utilities::createQueryFiltered($filters));
    }
    
    public function getTables($layoutId, $page, $pageSize, $filters){
        $finalPos = $pageSize * $page;
        $startPos = $finalPos - ($pageSize - 1);
        return $this->repositoryLayoutexp->getTables($layoutId, $startPos, $finalPos, Utilities::createQueryFiltered($filters));
    }
    
    public function getTablesLayoutExportation($filters){
        return $this->repositoryTablayoutexp->getTablesLayoutExportation(Utilities::createQueryFiltered($filters));
    }

    /**
     * @param $rows
     * @return array
     * @throws \Exception
     */
    public function saveLayoutTables($rows){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            $orgId = $this->environment->getCurrentOrganizationId();
            $savedIds = [];

            foreach($rows as $row) {
                $tableName   = $row['NAME'];
                $tableActive = $row['ACTIVE'];
                $layoutId    = $row['LAYOUT_ID'];
                $tableOrder  = $row['TABLE_ORDER'];

                $operatorConstraints = $this->repositoryLayoutExportation->getOperatorConstraintByTable($tableName, Utilities::getOrgDbUserName($orgId));

                if (!empty($operatorConstraints)) {
                    throw new \Exception(Messages::ERR_NOT_SAVE_TABLE_OPERATOR_CONSTRAINT);
                } else {
                    $dateNow = new \DateTime();
                    $userId = $this->environment->getCurrentUserId();

                    /** @var Integracao\Entities\Tablayoutexp $layoutTable */
                    $layoutTable = $this->repositoryTablayoutexp->findOneBy(array('cdlayoutexp' => $layoutId,
                        'nmtabelabd' => $tableName));
                    $savedLayoutTable = $this->factoryTableLayoutExportation->save($layoutTable, $userId, $dateNow, $orgId,
                        $layoutId, $tableName, $tableOrder, $tableActive);
                    $this->entityManager->persist($savedLayoutTable);
                    $savedIds[] = array("ID" => $savedLayoutTable->getNrseqtabexp());
                }
            }

            $this->entityManager->flush();
            $connection->commit();
            return $savedIds;
        } catch (\Exception $ex){
            $connection->rollBack();
            throw $ex;
        }
    }
}