module.exports = function(grunt){
    var utilPath = 'assets/js/Util/**/*.js';
    var servicePath = 'assets/js/Service/**/*.js';
    var controllerPath = 'assets/js/Controller/**/*.js';
    var libFiles = [
        'zhuaclfrontend/assets/dist/main.js',

        'zh-interval-component/dist/zh-interval-component.js',
        'zh-grid-navigation/dist/GridNavigationLibrary.js',
        'zh-memory-storage/src/main.js',
        "zh-cache-bypass/src/CacheBypass.js",
        'zh-select-autocomplete/dist/zh-select-autocomplete.js',
        'zh-reports/dist/zhReports.js',
        'zh-interval-component/dist/zh-interval-component.js',

        'zh-preferences/dist/preferences.js',
        'zh-filter-preferences/dist/filter-preferences.js',
        'zh-layout-preferences/dist/layout-preferences.js',
        'zh-view-preferences/dist/view-preferences.js',
        'zh-grid-configuration/dist/zh-grid-configuration.js',

        'zhulog/dist/main.js',
        'tek-libraries/dist/main.js'
    ];

    grunt.initConfig({
        // grunt-contrib-concat
        concat: {
            js: {
                options: {
                    process: function(src, filePath){
                        return '\n' + '// FILE: ' + filePath + '\n' + src;
                    }
                },
                src: [
                    utilPath, servicePath, controllerPath
                ],
                dest: 'assets/js/app.js'
            },
            css: {
                options: {
                    process: function (src, filepath) {
                        return '/* FILE: ' + filepath + ' */\n' + src + '\n';
                    }
                },
                src: getCSSPaths(),
                dest: './assets/css/app.css'
            },
            buildLibs: {
                options: {
                    process: function (src, filepath) {
                        return '// FILE: ' + filepath + '\n' + src + '\n';
                    }
                },
                src: getLibPaths(),
                dest: 'assets/js/libs.js'
            }
        },

        // grunt-contrib-jshint
        jshint: {
            options: {
                reporterOutput : ''
            },
            all: ['Gruntfile.js', './assets/js/Util/**/*.js', './assets/js/Service/**/*.js', './assets/js/Controller/**/*.js']
        },

        // grunt-contrib-uglify
        uglify: {
            options: {
                mangle: false,
                compress: false,
                report: 'min',
                banner: '/*! <%= grunt.template.today("dd-mm-yyyy") %> */\n'
            },
            dist: {
                files: {
                    'assets/js/app.min.js': [utilPath, servicePath, controllerPath],
                    'assets/js/environment.min.js': 'assets/js/environment.js',
                    'config/Config.min.js': 'config/Config.js',
                    'assets/js/libs.min.js': 'assets/js/libs.js',
                    'assets/js/engine.min.js': 'assets/js/engine.js',

                    'assets/codemirror/lib/codemirror.min.js': 'assets/codemirror/lib/codemirror.js',
                    'assets/codemirror/mode/php/php.min.js': 'assets/codemirror/mode/php/php.js',
                    'assets/codemirror/mode/javascript/javascript.min.js': 'assets/codemirror/mode/javascript/javascript.js',
                    'assets/codemirror/mode/xml/xml.min.js': 'assets/codemirror/mode/xml/xml.js',
                    'assets/codemirror/mode/clike/clike.min.js': 'assets/codemirror/mode/clike/clike.js',
                    'assets/codemirror/mode/css/css.min.js': 'assets/codemirror/mode/css/css.js',
                    'assets/codemirror/mode/sql/sql.min.js': 'assets/codemirror/mode/sql/sql.js',
                    'assets/codemirror/mode/shell/shell.min.js': 'assets/codemirror/mode/shell/shell.js',

                    'assets/codemirror/addon/hint/show-hint.min.js': 'assets/codemirror/addon/hint/show-hint.js',
                    'assets/codemirror/addon/hint/sql-hint.min.js': 'assets/codemirror/addon/hint/sql-hint.js',
                    'assets/codemirror/addon/display/placeholder.min.js': 'assets/codemirror/addon/display/placeholder.js',
                    'assets/codemirror/addon/display/fullscreen.min.js': 'assets/codemirror/addon/display/fullscreen.js',
                    'assets/codemirror/addon/search/search.min.js': 'assets/codemirror/addon/search/search.js',
                    'assets/codemirror/addon/search/searchcursor.min.js': 'assets/codemirror/addon/search/searchcursor.js',
                    'assets/codemirror/addon/search/jump-to-line.min.js': 'assets/codemirror/addon/search/jump-to-line.js',
                    'assets/codemirror/addon/edit/closebrackets.min.js': 'assets/codemirror/addon/edit/closebrackets.js',
                    'assets/codemirror/addon/edit/matchbrackets.min.js': 'assets/codemirror/addon/edit/matchbrackets.js'
                }
            }
        },

        cssmin: {
            dist: {
                files: {
                    'bower_components/zeedhi-frontend/assets/css/libs/bootstrap.min.css': 'bower_components/zeedhi-frontend/assets/css/libs/bootstrap.css',
                    'bower_components/zeedhi-frontend/assets/css/general.min.css': 'bower_components/zeedhi-frontend/assets/css/general.css',
                    'bower_components/zeedhi-frontend/assets/fonts/font-zeedhi/stylesheet.min.css': 'bower_components/zeedhi-frontend/assets/fonts/font-zeedhi/stylesheet.css',
                    'bower_components/zeedhi-frontend/assets/css/styles/styles-inflightor.min.css': 'bower_components/zeedhi-frontend/assets/css/styles/styles-inflightor.css',
                    
                    'bower_components/zh-grid-configuration/dist/style/colors/styles-inflightor.min.css': 'bower_components/zh-grid-configuration/dist/style/colors/styles-inflightor.css',
                    'bower_components/zh-preferences/dist/style/style.min.css': 'bower_components/zh-preferences/dist/style/style.css',
                    'bower_components/zh-preferences/dist/style/colors/styles-inflightor.min.css': 'bower_components/zh-preferences/dist/style/colors/styles-inflightor.css',
                    'bower_components/zh-grid-navigation/dist/style.min.css': 'bower_components/zh-grid-navigation/dist/style.css',
                    'bower_components/zh-select-autocomplete/dist/style/colors/styles-inflightor.min.css': 'bower_components/zh-select-autocomplete/dist/style/colors/styles-inflightor.css',

                    'assets/codemirror/lib/codemirror.min.css': 'assets/codemirror/lib/codemirror.css',
                    'assets/codemirror/addon/hint/show-hint.min.css': 'assets/codemirror/addon/hint/show-hint.css',
                    'assets/codemirror/addon/display/fullscreen.min.css': 'assets/codemirror/addon/display/fullscreen.css',
                    'assets/codemirror/theme/neat.min.css': 'assets/codemirror/theme/neat.css',
                    'assets/codemirror/theme/material.min.css': 'assets/codemirror/theme/material.css',
                    
                    'assets/css/app.min.css': 'assets/css/app.css'
                }
            }
        },

        // grunt-contrib-watch
        watch: {
            pivotal: {
                files: [utilPath, servicePath, controllerPath],
                tasks: ['jshint', 'uglify', 'concat:js']
            }
        },

        // grunt-zh-id-generator
        zhIdGenerator: {
            options: {
                jsonPath: ["./assets/json"],
                menuPath: ["./assets/json/menu.json"],
                replaceIds: false
            }
        },

        // grunt-zhuacl-structure
        buildAclStruct : {
            options: {
                menuConfigFile: "./menuConfig.json"
            }
        },

        buildEnvironment: {
            development: {
                configFile: "./config/devEnvironment.json"
            },
            production: {
                configFile: "./config/prodEnvironment.json"
            },
            test: {
                configFile: "./config/testEnvironment.json"
            },
            srcFolders: ["../backend/config/", "../backend/query/", "./assets/js", "./assets/json", "./config", "./", "./assets", "../version/"]
        }
    });

    function getLibPaths() {
        var paths = [];
        libFiles.forEach(function(fileName) {
            paths.push('bower_components/' + fileName);
        });
        return paths;
    }

    function getCSSPaths() {
        var paths = [];
        paths.push('./assets/css/**/*.css');
        paths.push('!./assets/css/app.css');
        paths.push('bower_components/zh-select-autocomplete/dist/style/colors/styles-inflightor.css');
        paths.push('bower_components/zh-grid-navigation/dist/style.css');
        paths.push('bower_components/zh-reports/dist/style/colors/styles-inflightor.css');
        paths.push('bower_components/zh-preferences/dist/style/style.css');
        paths.push('bower_components/zh-preferences/dist/style/colors/styles-inflightor.css');
        return paths;
    }

    grunt.loadNpmTasks('grunt-contrib-jshint');
    grunt.loadNpmTasks('grunt-contrib-uglify');
    grunt.loadNpmTasks('grunt-contrib-concat');
    grunt.loadNpmTasks('grunt-contrib-watch');
    grunt.loadNpmTasks('grunt-contrib-cssmin');
    grunt.loadNpmTasks('grunt-zh-id-generator');
    grunt.loadNpmTasks('grunt-zhuacl-structure');
    grunt.loadNpmTasks('grunt-zh-environment-builder');

    grunt.registerTask('default', ['jshint', 'concat', 'buildEnvironment', 'uglify', 'buildAclStruct', 'cssmin']);
    grunt.registerTask('buildLibs', ['concat:buildLibs']);
    grunt.registerTask('basic', ['uglify', 'concat:js', 'zhIdGenerator']);
};